/*
 * $Id: ClientGame.java,v 1.3 1998/04/07 12:17:41 vmallet Exp $
 *
 *
 * $Log: ClientGame.java,v $
 * Revision 1.3  1998/04/07 12:17:41  vmallet
 * First implementation of removePlayer();
 * Added the getPlayer(id) and the players table.
 * Added the dispose() method call for the players frame.
 *
 * Revision 1.2  1998/04/06 09:22:08  vmallet
 * Rewrote the ST_CREATE_PLAYER request handling code.
 * First try at displaying players in a window.
 *
 * Revision 1.1  1998/03/11 02:06:14  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.awt.*;
import java.awt.event.*;
import java.util.*; 

import com.sun.java.swing.*;

/**
 * The <code>ClientGame</code> class (not to be confused with 
 * <code>GameClient</code>) is the abstract class specifying what features 
 * a game that will be handled by the Jags client needs to provide.
 * <p>
 * [DESCRIBE ME MORE]
 * 
 * @version $Revision: 1.3 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public abstract class ClientGame implements GameLinkListener {

  /**
   * The <code>GameLinkFilter</code> object used to communicate with the Jags 
   * server.
   */
  protected GameLinkFilter gameLink;

  /**
   * The id of this game.
   */
  protected Object id;
  
  /**
   * The name of this game.
   */
  protected String name;
  
  /**
   * if true, enables the printing of debug informations on the console.
   */  
  protected boolean debug = true;

  
  /**
   * [DESCRIBE ME]
   */  
  Hashtable players = new Hashtable();
  

  /**
   * Creates a new ClientGame [DESCRIBE ME MORE]
   */
  public ClientGame() {
  }  
  
  /**
   * Creates a new ClientGame [DESCRIBE ME MORE]
   *
   * @param link the <code>GameLinkFilter</code> object used to communicate with
   *             the Jags server.
   */
  public ClientGame(GameLinkFilter link) {
    gameLink = link;

    link.addGameLinkListener(this, GAME_GROUP);
  }

  /**
   * Used as a substitution of the constructor which cannot be called with parameters
   * in a call to <code>java.lang.Class.newInstance()</code>. Actually constructs
   * the <code>ClientGame</code>.
   *
   * @param link       the link to the game in the server
   * @param myPlayerId the id of the player we're actually playing with.
   */
  public void make(GameLinkFilter link, int myPlayerId) {
    this.gameLink   = link;
    this.myPlayerId = myPlayerId;

    link.addGameLinkListener(this, GAME_GROUP);
  }


  /**
   * Creates a new particular <code>Player</code> for this game. This method 
   * should be overriden by children so the created player is of the right 
   * type.
   *
   * !@param name the name of the new player to create
   * @param id     the id of the player, which should have been sent by the server.
   * @param object the data needed to reconstruct the player in the client.
   * @return the newly created player of the right type to be used in this
   *         game.
   */
  public abstract Player createPlayer(int id, Object data);


  /**
   * Adds the given player to this game, as ordered by the server.
   *
   * @param player the player to add to the game.
   */
  public void addPlayer(Player player) {
    System.out.println("<ClientGame> addPlayer");

    //@@ VIOLENCE !
    gameLink.addGameLinkListener(player, PLAYER_GROUP);

    
    getPlayersPanel().add(player.getComponent());
    getPlayersPanel().validate();
    // getPlayersPanel().doLayout(); tiens, my fucking Ass !
    
    System.out.println("<ClientGame> panel = " + getPlayersPanel());
    
    Object[] comps = getPlayersPanel().getComponents();
    
    System.out.println("<ClientGame> panel.comps = <" + comps.length + ">" );
    for (int i = 0; i < comps.length; i++) 
      System.out.println("    - " + comps[i]);
    
    players.put(new Integer(player.getId()), player);


    //@@ do it
  }

  /**
   * Removes the player from this game, as ordered by the server.
   */
  public void removePlayer(Player player) {
    //@@ do it
    gameLink.deleteGameLinkListener(player, PLAYER_GROUP);
    players.remove(new Integer(player.getId()));
    getPlayersPanel().remove(player.getComponent());
    getPlayersPanel().validate();
  }
  

  /**
   * Returns the id of the game.
   */
  public Object getId() {
    return id;
  }
  
  /**
   * Sets the id of the game.
   */
  public void setId(Object id) {
    this.id = id;
  }

  /**
   * Sets the name of this game.
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * Returns the id of the player we're actually playing with.
   */
  public int getMyPlayerId() {
    return myPlayerId;
  }

  /**
   * 
   */  
  public Player getPlayer(int id) {
    return (Player) players.get(new Integer(id));
  }
  


  /**
   * Creates a 'default' window containing menus, toolbar, players and a panel
   * that can receive the game components. The window is stored and can be retrieved
   * through the <code>getComponent</code> method. The panel is returned.
   *
   * @param width  the width of the window
   * @param height the height of the window
   * @return the panel which can receive the game gui components.
   *
   * @see #getComponent
   */
  public Container createWindow(int width, int height) {
    jframe = new JFrame(name == null ? "A Jags Game" : name);
    jframe.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { doCloseGame(); }
    });

    menuBar = createMenuBar();
    jframe.setJMenuBar(menuBar);

    jframe.setSize(width, height);
    //    client.pack();
    jframe.setVisible(true);
    jframe.show();

    //    JPanel panel = new JPanel();
    
    jframe2 = new JFrame("players");
    jframe2.setSize(200, 300);
    jframe2.setVisible(true);
    jframe2.show();
    
    playersPanel = jframe2.getContentPane();
    playersPanel.setLayout(new BoxLayout(playersPanel, BoxLayout.Y_AXIS));

    return jframe.getContentPane();
  }

  /**
   * Creates the Menu Bar.
   */  
  public JMenuBar createMenuBar() {
    
    JMenuBar menuBar = new JMenuBar();

    JMenuItem item;
   
    // Create a menu labeled File, accelerator F
    
    // File Menu *****

    JMenu file = new JMenu("File");
    //    file.setKeyAccelerator('F');
    file.setMnemonic('F');
 
    file.addSeparator();
    
    // Quit item
    file.add(item = new JMenuItem("Quit"));
    item.setMnemonic ('Q');
    item.addActionListener (new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        doCloseGame();
      }
    });
   
    // Add file menu to menu bar
    menuBar.add (file);
    
    // Help Menu *****

    //@@ check that
    final String[] message = { "Jags",
			       "The Java Game System",
			       "\n",
			       "Written by :", 
			       "   Anne-Elizabeth Granier", 
			       "   Helene Legras",
			       "   Kristell Lozac'hmeur",
			       "   Frederique Poivert",
			       "   Loic Dunand",
			       "   Vincent Mallet",
			       "\n" };

    JMenu help = new JMenu("Help");
    help.setMnemonic('H');
    help.add(item = new JMenuItem("About.."));
    item.setMnemonic('A');
    item.addActionListener(new ActionListener() {
      public void actionPerformed (ActionEvent e) {
	JOptionPane.showMessageDialog(jframe, message,
				      "About", 
				      JOptionPane.INFORMATION_MESSAGE,
				      null
				      );
      }
    });
    
	
    // the setHelpMenu() is not yet implemented ! (vim 980209)
    // menuBar.setHelpMenu(help);
    menuBar.add(help);

    return menuBar;
  } // createMenuBar

  

  
  /**
   * Closes the game (and notifies the server)
   */  
  public void doCloseGame() {
    // notifies server
    LinkFrame frame = new LinkFrame();

    frame.setGroup(GAME_GROUP);
    frame.setRequest(RQ_LEAVE_GAME, myPlayerId);

    gameLink.sendBlindly(frame);
    
    jframe2.dispose();
    jframe.dispose();
  }


  /**
   * Returns the GUI component which is the graphical interface of the
   * game. Typically this component is a JFrame.
   *
   * @return the gui component beeing the graphical interface of the game.
   */  
  public Component getComponent() {
    return jframe;
  }

  /**
   * Returns the default menu bar.
   * @@experimental
   */  
  public JMenuBar getMenuBar() {
    return menuBar;
  }
  
  /**
   * Returns the default playersPanel
   */
  public Container getPlayersPanel() {
    return playersPanel;
  }

  
  // == implements GameLinkListener ============================
  
  public void frameReceived(Object link, LinkFrame frame) {
    if (debug)
      System.out.println("<ClientGame> frameReceived() " );
    
    int request = frame.getRequest();
    
    switch (request) {
    case ST_CREATE_PLAYER:
      if (debug)
	System.out.println("   - ST_CREATE_PLAYER");
      
//       Object[] data = (Object[]) frame.getData();

//       int playerId =  ((Integer) data[1]).intValue();
//       //@@ and create the first player (our player).
      
//       Player player = createPlayer((String) data[0], playerId);

      Object data     = frame.getData();
      int    playerId = frame.getSubRequest();
      Player player   = createPlayer(playerId, data);
      
      addPlayer(player);
      
      break;
    case ST_REMOVE_PLAYER:
      if (debug)
	System.out.println("  - ST_REMOVE_PLAYER: id=" + frame.getSubRequest());
      removePlayer(getPlayer(frame.getSubRequest()));
      break;

    default:
      if (debug)
	System.out.println("<ClientGame>   ! unhandled request");
      break;
    }
  }

  
  


  //== data =======================================================


  /**
   *
   */
  protected ClientGame myThis = this;


  /**
   * The default menu bar.
   */
  protected JMenuBar menuBar;

  /**
   * The window of this game.
   */
  protected JFrame jframe;

  /**
   * The window for the players - should be removed.
   */
  protected JFrame jframe2;


  /**
   * The Panel containing the players component
   */
  protected Container playersPanel;

  /**
   * Id of the player we're actually playing with.
   */
  protected int myPlayerId;

} // class ClientGame.
