/*
 * $Id: GameClient.java,v 1.4 1998/04/07 12:20:49 vmallet Exp $
 *
 *
 * $Log: GameClient.java,v $
 * Revision 1.4  1998/04/07 12:20:49  vmallet
 * Make use of the NamePanel object for player name prompting.
 *
 * Revision 1.3  1998/04/06 09:28:02  vmallet
 * Minor changes (window size, player random name etc).
 *
 * Revision 1.2  1998/03/11 02:12:05  vmallet
 * Added gameID support to frame code.
 * Other minor changes.
 *
 * Revision 1.1  1998/03/02 19:26:41  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.awt.*; 
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import com.sun.java.swing.plaf.*;
import com.sun.java.swing.tree.*;

import com.sun.java.swing.event.*;

import java.io.*;
import java.net.*; 
import java.util.*;

/**
 * The <code>GameClient</code> class is the main class of the <i>jags</i>
 * client. It contains the <code>main</code> method which
 * is used to start the client.<br>
 * <p>
 * This is a GUI oriented client for connecting to <i>jags</i> GameServer
 * servers. The interface is built using the <b>swing</b> classes from
 * <a href="http://java.sun.com">Sun Microsystems</a>.<br>
 *
 * <br> [FINISH ME]
 *
 * @see GameServer 
 *
 * @version $Revision: 1.4 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class GameClient extends JFrame implements GameLinkConstants {

  /** Frame width */
  static int WIDTH = 600;

  /** Frame height */
  static int HEIGHT = 300;

  /** internal constant */
  final static boolean DO_PROMPT = true;

  /** Class debug mode */
  final boolean debug = true;

  /**
   * Creates a new GameClient. Also creates all the gui components.
   *
   * @param title the gui window title.
   */
  public GameClient(String title) {   
    super(title);

    // Load images
    loadImages();

    Container content = getContentPane();

    // Menu stuff
    setJMenuBar(createMenuBar());

    // ToolBar stuff
    content.add(createToolBar(), BorderLayout.NORTH);

    // Servers stuff
    content.add(serverHandlers.getList(), BorderLayout.CENTER);
  }

  

  /**
   * Initiates a connection with a new server, on the specified port.
   * The server should be listening to this port (TCP connection).
   * The name and port informations can be prompted to the user using
   * the <code>connectDialog()</code> method.
   *
   * @param serverName the name or IP address of the server.
   * @param serverPort the port which the server listens to.
   *
   * @see #connectDialog
   */
  public void doOpenServer(String serverName, int serverPort) {
    if (debug)
      System.out.println("<GameClient> doOpenServer() - Should connect");

    //@@ a l'ultra arrache
    Socket socket = null;
    try {
      socket = new Socket(serverName, serverPort);
    } catch (Exception e) {
      //@@ Pop warning dialog here 
      System.out.println("Unable to open server on specified port!");
    }

    if (socket != null) {
      if (debug)
	System.out.println("<GameClient> doOpenServer() - "
			   + "Connection established, port: " 
			   + socket.getLocalPort());
      
      ServerHandler handler = null;
      try {
	handler = new ServerHandler(socket, serverName, serverPort);
      } catch (SocketException e) {
	//@@ do something appropriate
      }
	
      if (handler != null) {
	handler.start();
	serverHandlers.addServer(handler);
      }
    }
  } // doOpenServer




  /**
   * The GameClient entry point. Creates a new GameClient, sets it
   * up and starts the client interface.
   *
   * @param args the command line arguments passed to the method.
   */  
  public static void main(String args[]) {
    GameClient client = new GameClient("Jags Client 0.3-beta");
    client.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {System.exit(0);}
    });
    client.setSize(WIDTH, HEIGHT);
    //    client.pack();
    client.setVisible(true);
    client.show();
  }

  /**
   * Terminates the game client after having prompted for
   * confirmation.
   *
   * @param prompt whether we should prompt for confirmation.
   */  
  public void doTerminate(boolean prompt) {
    //@@ PROMPT STUFF
    System.exit(0);
  }


  // Private Area


  /**
   * Creates the Menu Bar.
   */  
  JMenuBar createMenuBar() {
    
    JMenuBar menuBar = new JMenuBar();

    JMenuItem item;
   
    // Create a menu labeled File, accelerator F
    
    // File Menu *****

    JMenu file = new JMenu("File");
    //    file.setKeyAccelerator('F');
    file.setMnemonic('F');
 
    // Open item
    file.add(item = new JMenuItem("Open"));
    item.setMnemonic ('O');
    item.addActionListener (new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        connectDialog();
      }
    });

    file.addSeparator();
    
    // Quit item
    file.add(item = new JMenuItem("Quit"));
    item.setMnemonic ('Q');
    item.addActionListener (new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        doTerminate(DO_PROMPT);
      }
    });
   
    // Add file menu to menu bar
    menuBar.add (file);
    
    // Help Menu *****

    final String[] message = { "Jags",
			       "The Java Game System",
			       "\n",
			       "Written by :", 
			       "   Anne-Elizabeth Granier", 
			       "   Helene Legras",
			       "   Kristell Lozac'hmeur",
			       "   Frederique Poivert",
			       "   Loic Dunand",
			       "   Vincent Mallet",
			       "\n" };

    JMenu help = new JMenu("Help");
    help.setMnemonic('H');
    help.add(item = new JMenuItem("About.."));
    item.setMnemonic('A');
    item.addActionListener(new ActionListener() {
      public void actionPerformed (ActionEvent e) {
	JOptionPane.showMessageDialog(myThis, message,
				      "About", 
				      JOptionPane.INFORMATION_MESSAGE,
				      aboutIcon
				      );
      }
    });
    
	
    // the setHelpMenu() is not yet implemented ! (vim 980209)
    // menuBar.setHelpMenu(help);
    menuBar.add(help);

    return menuBar;
  } // createMenuBar

  /**
   * Creates the ToolBar.
   */  
  JToolBar createToolBar() {
    JToolBar toolBar = new JToolBar();

    JButton button = new JButton(openIcon);
    button.setToolTipText("Open Server");
    button.setMargin(new Insets(0, 0, 0, 0));
    button.addActionListener(new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        connectDialog();
      }
    });
    toolBar.add(button);
    
    toolBar.addSeparator();

    button = new JButton("Close");
    button.setToolTipText("Close Server");
    button.setEnabled(false);
    button.addActionListener(new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        closeServerDialog();
      }
    });
    toolBar.add(button);
    serverCloseButton = button;

    toolBar.addSeparator();
    
    button = new JButton("Join");
    button.setToolTipText("Join Game");
    button.setEnabled(false);
    button.addActionListener(new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        joinGameDialog();
      }
    });
    toolBar.add(button);
    joinGameButton = button;

    toolBar.addSeparator();
    
    button = new JButton("Quit");
    button.setToolTipText("Quit Jags");
    button.addActionListener(new ActionListener() {
      public void actionPerformed (ActionEvent e) {
        doTerminate(DO_PROMPT);
      }
    });
    toolBar.add(button);


      
    

    // Make toolbar not floatable
    toolBar.setFloatable(false);

    return toolBar;
  } // createToolBar


  /**
   * Load images.
   */  
  public void loadImages() {
    Class myClass = getClass();
    openIcon  = new ImageIcon("images/open.gif"); //myClass.getResource("images/open.gif"));
    aboutIcon = new ImageIcon("images/DukeSnooze.gif"); //myClass.getResource("images/DukeSnooze.gif"));
  }


  /**
   * Pops a dialog and prompt user for a server to connect to.
   * Then calls the <code>doOpenServer()</code> method to do the
   * actual work.
   *
   * @see #doOpenServer
   */  
  public void connectDialog() {

    String  backupName = serverName;    // in case user aborts the dialog
    int     backupPort = serverPort;

    Border emptyBorder15 = new EmptyBorder(15,15,15,15);
    Border loweredBorder = new SoftBevelBorder(BevelBorder.LOWERED);

    Object[]  message    = new Object[3];

    JPanel mainp = new JPanel(true);
    mainp.setLayout(new BoxLayout(mainp, BoxLayout.Y_AXIS));
    
    Box        tfWrapper;
    JLabel     label;
    JTextField tf;
    
    label = new JLabel("host:");
    label.setHorizontalTextPosition(label.RIGHT);
    label.setDisplayedMnemonic('H');
    label.setToolTipText("Host name or IP address");

    tf = new JTextField(serverName);
    tf.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	serverName = ((JTextField) e.getSource()).getText();
      }
    });
    
    tfWrapper = new Box(BoxLayout.X_AXIS);
    tfWrapper.add(label);
    tfWrapper.add(Box.createHorizontalStrut(10));
    tfWrapper.add(tf);
    label.setLabelFor(tf);


    message[0] = "Select the Jags server to open             ";
    message[1] = tfWrapper;

    label = new JLabel("port:");
    label.setHorizontalTextPosition(label.RIGHT);
    label.setDisplayedMnemonic('p');
    label.setToolTipText("Host port number");

    tf = new JTextField(new Integer(serverPort).toString());
    tf.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	String           value = ((JTextField) e.getSource()).getText();
	int              newPort;
	
	try {
	  newPort = Integer.parseInt(value);
	} catch (Exception ex) {
	  newPort = -1;
	}
	if(newPort > 0)
	  serverPort = newPort;
	else
	  JOptionPane.showMessageDialog(myThis, "Invalid Port Number", 
					"Error", JOptionPane.ERROR_MESSAGE);
      }
    });
    
    tfWrapper = new Box(BoxLayout.X_AXIS);
    tfWrapper.add(label);
    tfWrapper.add(Box.createHorizontalStrut(10));
    tfWrapper.add(tf);
    label.setLabelFor(tf);

    message[2] = tfWrapper;
          
    String[]      options = { "Connect", "Cancel" };
    
    int res = JOptionPane.showOptionDialog(this, message, "Connect", 
					   JOptionPane.DEFAULT_OPTION, 
					   JOptionPane.PLAIN_MESSAGE, 
					   openIcon,
					   options, options[0]);

    if (res == 1) { // cancel
      serverName = backupName;
      serverPort = backupPort;
      return;
    }

    // connect

    doOpenServer(serverName, serverPort);
  } // connectDialog


  /**
   * 
   */  
  public void closeServerDialog() {
    ServerHandler server = (ServerHandler) serverHandlers.getSelectedObject();
    System.out.println("Should close connection with " + server);
    //@@ ultra-violence ! -vm930301
    server.close();
  }

  /**
   * 
   */
  public void joinGameDialog() {
    GameInfo info = (GameInfo) serverHandlers.getSelectedObject();
    
    ServerHandler server = 
      serverHandlers.getOwnerServer(serverHandlers.getSelectedNode());

    if (server != null)
      doGameConnect(info, server);
    else {
      if (debug)
	System.out.println("<GameClient> joinGameDialog(): server error");
    }
  }

  /**
   * [DESCRIBE ME]
   */
  public void doGameConnect(GameInfo info, ServerHandler server) {
    //@@ prompt for player name
    
    LinkFrame frame = new LinkFrame();
    Object[]  data  = new Object[2];

    data[0] = info.getGameId();
    //    data[1] = "toto" + Math.round(Math.random() * 100); //@@ player name
    data[1] = new NamePanel().getName();

    frame.setGameId(HANDLER_ID);
    frame.setGroup(HANDLER_GROUP);
    frame.setRequest(RQ_JOIN_GAME, NONE);
    frame.setData(data);

    try {
      server.getGameLink().send(frame);
    } catch (IOException e) {
      System.out.println("<ServerHandlers> doGameConnect xmit error");
    }
  }



  /**
   * Returns the toolbar ServerClose button
   */
  public JButton getServerCloseButton() {
    return serverCloseButton;
  }

  /**
   * Returns the toolbar JoinGame button.
   */
  public JButton getJoinGameButton() {
    return joinGameButton;
  }
  



  // Images used in the Client
  public ImageIcon openIcon;
  public ImageIcon aboutIcon;

  /** The Server Close Button */
  JButton serverCloseButton;

  /** The JoinGame Button */
  JButton joinGameButton;

  /** Default server name [PROPERTY ME] */
  String serverName = "localhost";

  /** Default server port [PROPERTY ME] */
  int    serverPort = 4213;
  
  /** this is us, the current GameClient instance */
  GameClient myThis = this;

  /**
   * List of Servers currently opened.
   */  
  ServerHandlers serverHandlers = new ServerHandlers(this);
}










/**
 * Holds the ServerHandlers, and provides a GUI tree representation
 * of all servers and games.
 */

class ServerHandlers extends Vector implements HandlerListener, GameLinkConstants {
  
  /**
   * This instance
   */
  ServerHandlers myThis = this;

  /**
   * The parent <code>GameClient</code>.
   */
  GameClient client;

  /**
   * The currently selected object.
   */
  DefaultMutableTreeNode selectedNode;
  
  /**
   * if true, enables the printing of debug informations on the console.
   */  
  boolean debug = true;

  
  /**
   * experimental
   */  
  class Tsl implements TreeSelectionListener {
    public void valueChanged(TreeSelectionEvent e) {
      TreePath p = e.getPath();
      DefaultMutableTreeNode node
	= (DefaultMutableTreeNode) p.getLastPathComponent();
      Object o = node.getUserObject();

      selectedNode = node;

      if (o instanceof GameInfo) {
	client.getJoinGameButton().setEnabled(true);
	client.getServerCloseButton().setEnabled(false);
      } else if (o instanceof ServerHandler) {
	client.getJoinGameButton().setEnabled(false);
	client.getServerCloseButton().setEnabled(true);
      } else {
	client.getJoinGameButton().setEnabled(false);
	client.getServerCloseButton().setEnabled(false);
      }
    }
  }


  /**
   * Creates a new ServerHandlers whose parent is the given <code>GameClient</code>.
   *
   * @param client the <code>GameClient</code> creating this 
   *        <code>ServerHandlers</code>.
   *
   * @see GameClient
   */
  public ServerHandlers(GameClient client) {
    this.client = client;
    //    pane = new ScrollPane();
    list = new JList();
    //    pane.add(list, BorderLayout.CENTER);
    //    pane.validate();
    
    // Create root node of tree
    DefaultMutableTreeNode root = 
      new DefaultMutableTreeNode("Servers");

    serversRoot = root;

    treeModel = new DefaultTreeModel(root);
    treeModel.setAsksAllowsChildren(true);

    // Create a tree from the root
    tree = new JTree(treeModel);

    //@@ experimental
    MouseListener ml = new MouseAdapter() {
      public void MouseClicked(MouseEvent e) {
	System.out.println( "sdfsdf");
	TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
	if (selPath != null)
	  System.out.println("Yope. got a path");
      }
      public void MouseEntered(MouseEvent e) {
	System.out.println( "sdddd--dfsdf");
	TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
	if (selPath != null)
	  System.out.println("Yope. got a path" + selPath);
      }
      public void MouseReleased(MouseEvent e) {
	System.out.println( "sdfsdf");
	TreePath selPath = tree.getPathForLocation(e.getX(), e.getY());
	if (selPath != null)
	  System.out.println("Yope. got a path");
      }
    };
    //@@ experimental
    tree.addMouseListener(ml);
    
    //@@ experimental
    tree.addTreeSelectionListener(new Tsl());

    // Place tree in JScrollPane
    pane = new JScrollPane();
    pane.getViewport().add (tree);
    pane.getViewport().addMouseListener(ml);
    pane.addMouseListener(ml);

    
    
    //    add(pane, BorderLayout.CENTER);
    //    pane.add(tree, BorderLayout.CENTER);
  }
    

  /**
   * Adds a server in the ServerHandler list.
   * 
   * @param server the server to add to the set.
   */
  public void addServer(ServerHandler server) {
    addElement(server);
    list.setListData(this);
    server.addListener(this);
    //    pane.validate();
    //    list.repaint();

    // Create 1st level child
    DefaultMutableTreeNode level1 = 
      new DefaultMutableTreeNode(server);

    treeModel.insertNodeInto(level1, serversRoot, 0);
    
//     serversRoot.add(level1);
//     pane.getViewport().remove(tree);
//     pane.getViewport().add(new JTree(serversRoot, true));
//     pane.repaint();
    //    tree = new JTree(serversRoot);
    
    
    //    tree.treeDidChange();
    //    System.out.println("<ServerHandlers> addServer() ");
  }

  /**
   * Messaged when the server data are modified (ie name, available
   * games and so on).
   *
   * @param server the server whose data changed.
   *
   * @see HandlerListener
   */ 
  public void handlerHasChanged(Handler _server) {
    //    list.setListData(this);
    //    list.repaint();
    // tree.invalidate();

    ServerHandler server = (ServerHandler) _server;
    int status = server.getChangeStatus();

    if (status == ServerHandler.NO_CHANGE) //@@ ? !!
      return; 
    
    DefaultMutableTreeNode serverNode = getObjectNode(serversRoot, server);

    if (status == ServerHandler.INFO_CHANGED) {
      GameInfo info = server.getChangeInfo();

      DefaultMutableTreeNode gameNode = getObjectNode(serverNode, info);
      if (gameNode == null) {
	if (debug)
	  System.out.println("<ServerHandlers> handlerHasChanged: game error");
      } else {
	treeModel.nodeChanged(gameNode);
      }

      return;
    }

    if (status == ServerHandler.INFO_REMOVED) {
      GameInfo info = server.getChangeInfo();
      
      DefaultMutableTreeNode gameNode = getObjectNode(serverNode, info);
      if (gameNode == null) {
	if (debug)
	  System.out.println("<ServerHandlers> INFO_REMOVED: game error");
      } else {
	treeModel.removeNodeFromParent(gameNode);
      }
      treeModel.nodeChanged(serverNode);
      
      return;
    } 

    treeModel.nodeChanged(serverNode);

    for (Enumeration e = ((ServerHandler)server).getGameInfos();
	      e.hasMoreElements(); ) {
      GameInfo info = (GameInfo) e.nextElement();
      DefaultMutableTreeNode gameNode = getObjectNode(serverNode, info);
      if (gameNode == null) {
	System.out.println("adding game, class=" 
			   + info.getClientGameClassName());
	gameNode = new DefaultMutableTreeNode(info);
	if (info.isNewGame())
	  gameNode.setAllowsChildren(false);
	treeModel.insertNodeInto(gameNode, serverNode, 0);
      }
    }

    //    pane.repaint();
  }

  /**
   * Messaged when the connection with server is lost.
   *
   * @param server the server from which we lost connection.
   *
   * @see HandlerListener
   */ 
  public void handlerLostConnection(Handler server) {
    removeElement((ServerHandler) server);
    list.setListData(this);
    //    list.repaint();

    DefaultMutableTreeNode node = getObjectNode(serversRoot, server);

    if (node != null) 
      treeModel.removeNodeFromParent(node);
  }

  /**
   * Search in the set of children of parent the child containing a given
   * object.
   *
   * @param root the parent node.
   * @param searched the searched object.
   * @return the child node containing the searched object, <code>null</code>
   *         if not found.
   */  
  public DefaultMutableTreeNode getObjectNode (
     DefaultMutableTreeNode parent, Object searched) {
    for (Enumeration e = parent.children(); e.hasMoreElements(); ) {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode) e.nextElement();
      if (node.getUserObject() == searched)
	return node;
    }
    return null;
  }


  /**
   * Returns the currently selected object in the tree.
   */
  public Object getSelectedObject() {
    return selectedNode.getUserObject();
  }

  /**
   * Returns the currently selected tree node.
   */
  public DefaultMutableTreeNode getSelectedNode() {
    return selectedNode;
  }

  /**
   * Returns the <code>ServerHandler</code> owning the given <code>GameInfo</code>,
   * or <code>null</code> if no corresponding server could be found.
   *
   * @param info the info for which the parent server has to be returned.
   * @return the server owning this game.
   */
  public ServerHandler getOwnerServer(DefaultMutableTreeNode node) {
    for (node = (DefaultMutableTreeNode) node.getParent(); node != null;
	 node = (DefaultMutableTreeNode) node.getParent()) {
      if (node.getUserObject() instanceof ServerHandler)
	return (ServerHandler) node.getUserObject();
    }
    
    return null;
  }




  /**
   * [DESCRIBE ME]
   *
   * @see GameClient#doGameConnect
   */
  public void doGameConnect(GameInfo info, ServerHandler server) {
    client.doGameConnect(info, server);
  }


  /**
   * GUI stuff
   */
  public JScrollPane getList() {
    return pane;
  }
  
  /** root node of the tree */
  DefaultMutableTreeNode serversRoot;

  /** tree data model */
  DefaultTreeModel treeModel;

  /** The MVC tree delegate wrapping around the tree data model */
  JTree tree;

  /**   */  
  JList list;

  /**   */
  JScrollPane pane;
} // ServerHandlers


