/*
 * $Id: GameInfo.java,v 1.2 1998/03/11 01:55:36 vmallet Exp $
 *
 *
 * $Log: GameInfo.java,v $
 * Revision 1.2  1998/03/11 01:55:36  vmallet
 * Added the clientGameClassName code.
 *
 * Revision 1.1  1998/03/02 19:28:23  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.io.*;


/**
 * GameInfo.java
 * 
 * @version $Revision: 1.2 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class GameInfo implements Serializable {

  /**
   * true if this is a new game, false for a played game.
   */  
  boolean newGame;

  /**
   * Name of the game.
   */
  String name;

  /**
   * Name of the class that has to be used in the Jags client
   * to be able to connect to this game in the Jags server.
   */
  String clientGameClassName;

  /** 
   * Unique ID of the game
   */
  Object id;

  /**
   * Maximum number of players in this game.
   */
  int    maximumPlayers;

  /**
   * Number of players currently playing this game (if
   * not a new game).
   */
  int    numPlayers;

  /**
   * Creates a new GameInfo related to a new game.
   */
  public GameInfo(Game game) {
    newGame             = true;
    this.id             = game.getId();
    name                = game.getName();
    maximumPlayers      = game.getMaximumPlayers();
    clientGameClassName = game.getClientGameClassName();
  }

  /**
   * Creates a new GameInfo
   */
  public GameInfo(GameHandler runningGame) {
    this(runningGame.getGame());
    newGame = false;

    numPlayers = runningGame.getPlayerCount();
  }


  /**
   * Returns whether this is a new game.
   * @return true if it is a new game, false if it is a running game.
   */
  public boolean isNewGame() {
    return newGame;
  }

  
  /**
   * Returns the name of the game.
   */
  public String getGameName() {
    return name;
  }

  /**
   * Returns the name of the class that has to be used in the Jags client
   * to be able to connect to this game in the Jags server.
   */
  public String getClientGameClassName() {
    return clientGameClassName;
  }

  /**
   * Returns the id of the game.
   */
  public Object getGameId() {
    return id;
  }

  /**
   * Sets the id of the game.
   */
  public void setGameId(Object id) {
    this.id = id;
  }


  /**
   * Updates the current instance of <code>GameInfo</code> with data
   * contained in the given other <code>GameInfo</code>. Everything is
   * copied from the other object but the id that will remain unchanged.
   *
   * @param other the object containing the data from which to update.
   */
  public void update(GameInfo other) {
    //@@ ultra-violence, yeaaaaaaahhhhhh... not :( -vm980301
    newGame             = other.newGame;
    name                = other.name;
    numPlayers          = other.numPlayers;
    maximumPlayers      = other.maximumPlayers;
    clientGameClassName = other.clientGameClassName;
  }
  

  /**
   * Returns a string representation of the object.
   */
  public String toString() {
    String result;

    if (newGame) {
      result = "New Game : " + name + "   (" 
	+ maximumPlayers + " players max)";
    } else {
      result = "Running Game : " + name + "   ("
	+ numPlayers + " player" + ((numPlayers > 1) ? "s" : "") 
        + ", " + maximumPlayers + " max)";
    }

    return result;
  }
}
