/*
 * $Id: GameLink.java,v 1.3 1998/04/06 19:18:27 vmallet Exp $
 *
 *
 * $Log: GameLink.java,v $
 * Revision 1.3  1998/04/06 19:18:27  vmallet
 * Lowered shouldQuit limit from 8 to 4.
 *
 * Revision 1.2  1998/03/11 02:09:55  vmallet
 * Changed the listener code so that it depends on the game ID
 * instead of the frame group.
 *
 * Revision 1.1  1998/03/02 19:28:31  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.io.*;
import java.util.*; 
import java.net.*; 

/**
 * The <B>GameLink</B> class implements the actual connection 
 * between the Server and its Clients. It is based on the use
 * of reliable Sockets. [FINISH ME]
 * 
 * @version $Revision: 1.3 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public class GameLink extends Thread implements GameLinkConstants {

  /**
   * The socket used for the link.
   */
  Socket socket;

  /**   */  
  InputStream is;

  /** */
  OutputStream os;

  /** 
   * if true, enables the printing of debug informations on the console.
   */
  boolean debug = true;
  

  /**
   * Creates a new GameLink, wrapping around the given Socket.
   * @param socket the Socket to be used for data exchange on the link.
   */
  public GameLink(Socket socket) throws SocketException { 
    this.socket = socket;
    socket.setSoTimeout(SOCKET_TIMEOUT);
    try {
      is = socket.getInputStream();
      os = socket.getOutputStream();
    } catch (IOException e) {
      throw new SocketException(e.toString());
    }
    connected = true;
  }

  /**
   * Tells whether the connection is still alive.
   */  
  public boolean isConnected() {
    synchronized (this) {
      return connected;
    }
  }

  /**
   * experimental.
   *
   * @exception IOException thrown if an error occured during the sending 
   *            of the frame 
   * @see #sendBlindly
   */  
  public void send(LinkFrame frame) throws IOException {
    //@@
    if (frame.getGameId() == null) 
      System.out.println("send: here - null frame send");
    new ObjectOutputStream(os).writeObject(frame);
  }

  /**
   * experimental.
   *
   * @see #send
   */  
  public void sendBlindly(LinkFrame frame) {
    try { 
      //@@
      if (frame.getGameId() == null) 
	System.out.println("sendBlindly: here - null frame send");
      new ObjectOutputStream(os).writeObject(frame);
    } catch (IOException e) {
      System.out.println("<GameLink> sendBlindly(): IoException: " + e);
    }
  }

  /**
   * experimental.
   *
   * @exception ClassNotFoundException FATAL - the class corresponding
   * to the object read on the <code>ObjectInputStream</code> cannot
   * be found in the current JVM.
   */  
  public LinkFrame receive() throws IOException, ClassNotFoundException {
    return (LinkFrame) new ObjectInputStream(is).readObject();
  }



  /**
   * Close this <code>GameLink</code> link.
   */
  public void close() {
    try {
      socket.close(); //@@ check for synchronization.
    } catch (IOException e) {
      // muffle exception.. :)
    }
  }

  /**
   * Run! Run! [DESCRIBE ME]
   */  
  public void run() {
    int shouldQuit = 0; // a ultra revoir. juste pour mettre en place.
    while (shouldQuit < 4) {
      try {
	LinkFrame frame = receive();
	if (debug)
	  System.out.println("<GameLink> run() - Frame received: " + frame);
	notifyListeners(frame);
	shouldQuit = 0;
      } catch (SocketException e) {
	//	System.out.println("Frame SEVERE Exception: " + e);
	shouldQuit++;
      } catch (InterruptedIOException e) {
	// 	System.out.println("Frame ok Exception: " + e);
	shouldQuit = 0;
      } catch (IOException e) {
	if (debug)
	  System.out.println("<GameLink> run() - Frame IO Exception: " + e);
	shouldQuit++;
      } catch (ClassNotFoundException e) {
	throw new RuntimeException(e.toString());
      }
    }
    // here we've lost the connection !
    synchronized (this) {
      connected = false;
    }
  }

  /**
   * Adds the specified listener to the sets of listeners of this <code>GameLink</code>
   * object. The listener will listen only to the messages related to the specified
   * game.
   *
   * @param listener the specified listener to be added.
   * @param gameId   the id of the game of which the listener wants to get the messages.
   */  
  public synchronized void addGameLinkListener(GameLinkListener listener, 
					       Object gameId) {
    //    Integer groupInt = new Integer(group);
    Vector  groupListeners;

    if (listeners.containsKey(gameId))
      groupListeners = (Vector) listeners.get(gameId);
    else {
      groupListeners = new Vector();
      listeners.put(gameId, groupListeners);
    }
    
    groupListeners.addElement(listener);
  }
  

  /**
   * Removes the specified listener from the sets of listeners of this 
   * <code>GameLink</code> object for the given game id.
   * 
   * @param listener the listener to be removed from the set.
   * @param gameId   the id of the game set.
   */
  public synchronized void deleteGameLinkListener(GameLinkListener listener, 
						  Object gameId) {
    Vector groupListeners = (Vector) listeners.get(gameId);
    //@@ we might want to do something if the group does not exist. -vm 980308
    if (groupListeners != null)
      groupListeners.removeElement(listener);
  }

  /**
   * experimental.
   */  
  void notifyListeners(LinkFrame frame) {
    //    Integer group = new Integer(frame.getGroup());
    Object  gameId = frame.getGameId();

    if (gameId == null) {
      if (debug)
	System.out.println("<GameLink> frame contained no gameId ! Ignoring it.");
      return;
    }

    Vector groupListeners = (Vector) listeners.get(gameId);

    if (groupListeners != null) {
      for (Enumeration f = groupListeners.elements(); f.hasMoreElements(); ) {
	GameLinkListener listener = (GameLinkListener) f.nextElement();
	listener.frameReceived(this, frame);
      }
    }
  }


  // Private Area

  /**   */  
  Hashtable listeners = new Hashtable();

  /**   */  
  boolean connected;
}
