/*
 * $Id: GameLinkConstants.java,v 1.5 1998/04/07 12:14:53 vmallet Exp $
 *
 *
 * $Log: GameLinkConstants.java,v $
 * Revision 1.5  1998/04/07 12:14:53  vmallet
 * Added ST_REMOVE_PLAYER.
 *
 * Revision 1.4  1998/04/06 09:15:34  vmallet
 * Added ST_CREATE_PLAYER.
 *
 * Revision 1.3  1998/03/11 02:12:39  vmallet
 * Added a few constants.
 *
 * Revision 1.2  1998/03/06 20:34:09  vmallet
 * Added the GAME_GROUP.
 *
 * Revision 1.1  1998/03/02 19:28:31  vmallet
 * Initial revision
 *
 *
 */ 

package game;

/**
 * The <code>GameLinkConstants</code> interface contains all the 
 * constants the <code>GameLink</code> class and other related classes
 * need to operate properly.
 * 
 * @see GameLink
 * @version $Revision: 1.5 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public interface GameLinkConstants {

  /** */
  public static final String HANDLER_ID  = "H@NDLER";
      
  
  /** Frame group : lower level communication between GameLink objects */  
  public static final int SERVICE_GROUP  = 0;
  
  /** Frame group : for low level communication between Handler objects */  
  public static final int HANDLER_GROUP  = 1;

  /** Frame group : for communication between Player objects */
  public static final int PLAYER_GROUP   = 20;
  
  /** Frame group : for communication between Game objects */
  public static final int GAME_GROUP     = 40;


  /** The Socket timeout in milliseconds  */  
  public static final int SOCKET_TIMEOUT = 1000;


  /** None, nothing, niet, nada. Usually used for subrequests */  
  public static final int NONE           = 0;


  /** Request message : get informations about the server */  
  public static final int RQ_SERVERID    = 10;

  /** Request message : join a game (new or running) */  
  public static final int RQ_JOIN_GAME   = 20;

  /** Request message : leave a game */  
  public static final int RQ_LEAVE_GAME  = 30;


  public static final int RQ_PLAYERCHANGENAME   = 100;
  public static final int RQ_PLAYERNEWSCORE     = 110;
  public static final int RQ_PLAYERCHANGELETTER = 120;

  

  /** Server Status message : a game has been added */
  public static final int ST_GAME_ADDED    = 500;

  /** Server Status message : a game has been removed */
  public static final int ST_GAME_REMOVED  = 510;

  /** Server Status message : informations about a game changed */
  public static final int ST_INFO_CHANGED  = 520;

  /** Command Message : client must create an instance of the specified game */
  public static final int ST_CREATE_GAME   = 530;
  
  /** Command Message : client must create a player in the specified game */
  public static final int ST_CREATE_PLAYER = 540;

  /** Command Message : client must remove a player from the specified game */
  public static final int ST_REMOVE_PLAYER = 550;
   
  
  /* Reply : the game requested by the client 
   * cannot be found in the server
   */
  public static final int REPLY_INVALID_GAME_ID  = 1000;

  /** */
  public static final int REPLY_TOO_MANY_PLAYERS = 1010;
  
  /** */
  public static final int REPLY_CLOSED_GAME      = 1020;
}
