/*
 * $Id: GameLinkListener.java,v 1.2 1998/03/11 02:07:25 vmallet Exp $
 *
 *
 * $Log: GameLinkListener.java,v $
 * Revision 1.2  1998/03/11 02:07:25  vmallet
 * Added the "gameLink" parameter to the frameReceived() method.
 *
 * Revision 1.1  1998/03/02 19:28:31  vmallet
 * Initial revision
 *
 *
 */

package game;

/**
 * The <code>GameLinkListener</code> interface should be implemented by 
 * classes whose instances wants to be notified when a new frame is
 * received by the GameLink object. Objects register with the <code>
 * GameLink</code> object by calling the <code>addGameLinkListener</code.
 * Frames carried by the <code>GameLink</code> object are classified
 * in <i>groups</i>, and an object registers for listening for one group
 * of frames at a time, so that not all objects listens to all frames.
 *
 * @see GameLink
 * @see GameLink#addGameLinkListener
 * @version $Revision: 1.2 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public interface GameLinkListener extends GameLinkConstants {

  /**
   * Messaged when a new data frame is received by the <code>GameLink</code>
   * object and this frame belongs to the groups the GameLinkListener
   * asked to listen to.
   *
   * @param gameLink  the GameLink that actually received the frame.
   * @param frame     the frame that has just been received.
   *
   * @see GameLink
   */  
  public void frameReceived(Object gameLink, LinkFrame frame);
}
