/*
 * $Id: Handler.java,v 1.2 1998/03/11 02:05:11 vmallet Exp $
 *
 *
 * $Log: Handler.java,v $
 * Revision 1.2  1998/03/11 02:05:11  vmallet
 * Changed HANDLER_GROUP to HANDLER_ID in constructor.
 *
 * Revision 1.1  1998/03/02 19:28:31  vmallet
 * Initial revision
 *
 *
 */

package game;

import java.net.*;
import java.util.*;

/**
 * The ... [DESCRIBE ME]
 * 
 * @version $Revision: 1.2 $
 * @author  Vincent Mallet (vmallet@enst.fr)
 */
public abstract class Handler extends Thread implements GameLinkListener {

  /**
   * The <code>GameLink</code> object is used by the handler on one side (ie
   * server) to communicate with the other handler on the other side (ie client).
   */  
  GameLink gameLink;

  /**
   * if true, enables the printing of debug informations on the console.
   */  
  boolean debug = true;


  /**
   * Creates a new Handler
   */
  public Handler(Socket socket) throws SocketException {   
    gameLink = new GameLink(socket);
    //    gameLink.addGameLinkListener(this, HANDLER_GROUP);
    gameLink.addGameLinkListener(this, HANDLER_ID);
    gameLink.start();
  }

  /**
   * Close this handler, by simply shutting down the <code>GameLink</code>
   * connection. Derivated classes should override this method to have
   * a better control of the close operation.
   *
   * @see GameLink#close
   */
  public void close() {
    gameLink.close();
  }
  

  /**
   * Returns GameLink object. [CHECK THIS]
   */
  public GameLink getGameLink() {
    return gameLink;
  }


  /**
   * Adds a listener to the set of listeners for this Handler. 
   *
   * @param   hl   a listener to be added.
   */
  public synchronized void addListener(HandlerListener hl) {
    if (!listeners.contains(hl)) {
      listeners.addElement(hl);
    }
  }

  /**
   * Deletes a listener from the set of listeners of this Handler. 
   *
   * @param   hl   the listener to be deleted.
   */
  public synchronized void deleteListener(HandlerListener hl) {
    listeners.removeElement(hl);
  }


  /**
   * Returns the number of listeners of this object.
   *
   * @return  the number of listeners of this object.
   */
  public synchronized int countListeners() {
    return listeners.size();
  }

  
  /**
   * experimental
   */
  public void notifyListeners() {
    
    int size=0;
    HandlerListener[] arr;
    
    synchronized (this) {
      size = listeners.size();
      arr = new HandlerListener[size];
      listeners.copyInto(arr);
    }
    
    for (int i = size - 1; i>=0; i--) {
      if (arr[i] != null) {
	arr[i].handlerHasChanged(this);
      }
    }
  }
  

  /**
   * experimental
   */
  public void lostConnectionNotify() {
    
    int size=0;
    HandlerListener[] arr;
    
    synchronized (this) {
      size = listeners.size();
      arr = new HandlerListener[size];
      listeners.copyInto(arr);
    }
    
    for (int i = size - 1; i>=0; i--) {
      if (arr[i] != null) {
	arr[i].handlerLostConnection(this);
      }
    }
  }



  /**
   * Pads the given string to the given size with white spaces.
   *
   * @param s the string to pad
   * @param size the size of the new string returned
   * @return the string space-padded.
   */
  public String pad(String s, int size) {
    return (s + "                                                  " +
	    "                                                  " +
	    "                                                  "
	    ).substring(0, size);
  }

  /**
   * @@ has to be revised -vm980301
   */
  public void addManagedPlayer(Player player, GameHandler handler) {    
    Vector gameHandlers = (Vector) players.get(player);

    if (gameHandlers == null) {
      gameHandlers = new Vector();
      players.put(player, gameHandlers);
    }
    gameHandlers.addElement(handler);
  }

  /**
   * @@ has to be revised -vm980301
   */
  public Enumeration getManagedPlayers() {
    return players.keys();
  }

  /**
   * @@ has to be revised -vm980301
   */
  public Enumeration getPlayerGames(Player player) {
    return ((Vector) players.get(player)).elements();
  }


  /**
   * @@ has to be revised -vm980301
   */
  public void removePlayer(Player player) {
    System.out.println("<Handler> removePlayer(): not yet implemented");
  }

  /**
   * @@ has to be revised -vm980301
   */
  public void removePlayer(Player player, Game game) {
    System.out.println("<Handler> removePlayer(): not yet implemented");
  }





  /** Set of listeners of this Handler */  
  Vector listeners = new Vector();

  /**
   * Set of players managed by this handler. [FINISH ME]
   */
  Hashtable players = new Hashtable();
}


